<?php
class Page{

	// database connection and table title
	private $conn;
	private $table_name = "pages";

	// object properties
	public $id;
	public $title;
	public $slug;
	public $body;
	public $meta_description;
	public $status;
	public $featured_image;
	public $user_id;
	public $firstname;
	public $lastname;
	public $created;
	public $modified;

	public function __construct($db){
		$this->conn = $db;
	}

	public function removeFeaturedImage(){

		// get featured image file name for deletion on server directory
		$this->readOne();

		// empty featured image field
		$query = "UPDATE " . $this->table_name . "
				SET
					featured_image=''
				WHERE id = :id";

		$stmt = $this->conn->prepare($query);

		// sanitize
		$this->id=htmlspecialchars(strip_tags($this->id));

		// bind values
		$stmt->bindParam(':id', $this->id);

		// execute the query
		if($stmt->execute()){

			// delete image from server directory
			if(!unlink("../images/" . $this->featured_image)){
				echo "<div class='alert alert-danger'>Unable to delete file from server. $this->featured_image</div>";
			}

			// to tell the user featured image filed was emptied
			return true;
		}else{
			return false;
		}
	}

	// check if given slug exists in the database
	public function slugExists(){

		// query to check if slug exists
		$query = "SELECT id
				FROM " . $this->table_name . "
				WHERE slug = :slug AND id != :id
				LIMIT 0,1";

		// prepare the query
		$stmt = $this->conn->prepare( $query );

		// sanitize
		$this->slug=htmlspecialchars(strip_tags($this->slug));
		$this->id=htmlspecialchars(strip_tags($this->id));

		// bind given slug value
		$stmt->bindParam(':slug', $this->slug);
		$stmt->bindParam(':id', $this->id);

		// execute the query
		$stmt->execute();

		// get number of rows
		$num = $stmt->rowCount();

		// if slug exists
		if($num>0){

			// return true because slug exists in the database
			return true;
		}

		// return false if slug does not exist in the database
		return false;

	}

	// create page
	function create(){

		//write query
		$query = "INSERT INTO
					" . $this->table_name . "
				SET
					title=:title,
					slug=:slug,
					body=:body,
					meta_description=:meta_description,
					featured_image=:featured_image,
					status=:status,
					user_id=:user_id,
					created=:created";

		$stmt = $this->conn->prepare($query);

		// sanitize
		$this->title=htmlspecialchars(strip_tags($this->title));
		$this->slug=htmlspecialchars(strip_tags($this->slug));
		$this->body=htmlspecialchars($this->body);
		$this->meta_description=htmlspecialchars(strip_tags($this->meta_description));
		$this->featured_image=htmlspecialchars(strip_tags($this->featured_image));
		$this->status=htmlspecialchars(strip_tags($this->status));
		$this->user_id=htmlspecialchars(strip_tags($this->user_id));

		// bind values
		$stmt->bindParam(':title', $this->title);
		$stmt->bindParam(':slug', $this->slug);
		$stmt->bindParam(':body', $this->body);
		$stmt->bindParam(':meta_description', $this->meta_description);
		$stmt->bindParam(':featured_image', $this->featured_image);
		$stmt->bindParam(':status', $this->status);
		$stmt->bindParam(':user_id', $this->user_id);

		$this->created = date('Y-m-d H:i:s');
		$stmt->bindParam(':created', $this->created);

		if($stmt->execute()){
			return true;
		}else{
			echo "<pre>";
				print_r($stmt->errorInfo());
			echo "</pre>";
			return false;
		}

	}

	// read pages
	function readAll_PublishedPages(){

		$query = "SELECT
					p.id, p.user_id, p.title, p.slug, p.body,
					p.meta_description, p.status, p.featured_image,
					p.created, u.firstname, u.lastname
				FROM " . $this->table_name . " p
					LEFT JOIN users u
						ON p.user_id = u.id
				WHERE p.status = 1
				ORDER BY p.id DESC";

		// prepare query statement
		$stmt = $this->conn->prepare($query);

		// execute query
		$stmt->execute();

		return $stmt;
	}

	// read pages
	function readAll($from_record_num, $records_per_page){

		$query = "SELECT
					p.id, p.user_id, p.title, p.slug, p.body,
					p.meta_description, p.status, p.featured_image,
					p.created, u.firstname, u.lastname
				FROM " . $this->table_name . " p
					LEFT JOIN users u
						ON p.user_id = u.id
				ORDER BY p.id DESC
				LIMIT ?, ?";

		// prepare query statement
		$stmt = $this->conn->prepare($query);

		// bind limit clause variables
		$stmt->bindParam(1, $from_record_num, PDO::PARAM_INT);
		$stmt->bindParam(2, $records_per_page, PDO::PARAM_INT);

		// execute query
		$stmt->execute();

		return $stmt;
	}

	// used for paging pages
	public function countAll(){

		$query = "SELECT id FROM " . $this->table_name . "";

		$stmt = $this->conn->prepare( $query );
		$stmt->execute();

		$num = $stmt->rowCount();

		return $num;
	}

	// search pages
	function search($search_term, $from_record_num, $records_per_page){

		$query = "SELECT
					p.id, p.user_id, p.title, p.slug, p.body,
					p.meta_description, p.status, p.featured_image,
					p.created, u.firstname, u.lastname
				FROM " . $this->table_name . " p
					LEFT JOIN users u
						ON p.user_id = u.id
				WHERE
					p.title LIKE ?
					OR p.slug LIKE ?
					OR p.body LIKE ?
					OR p.meta_description LIKE ?
				ORDER BY p.id DESC
				LIMIT ?, ?";

		// prepare query statement
		$stmt = $this->conn->prepare($query);

		// sanitize
		$search_term = "%{$search_term}%";
		$search_term=htmlspecialchars(strip_tags($search_term));

		// bind limit clause variables
		$stmt->bindParam(1, $search_term);
		$stmt->bindParam(2, $search_term);
		$stmt->bindParam(3, $search_term);
		$stmt->bindParam(4, $search_term);
		$stmt->bindParam(5, $from_record_num, PDO::PARAM_INT);
		$stmt->bindParam(6, $records_per_page, PDO::PARAM_INT);

		// execute query
		$stmt->execute();

		return $stmt;
	}

	// used for paging pages
	public function countAll_BySearch($search_term){

		$query = "SELECT id FROM " . $this->table_name . "
					WHERE
						title LIKE ?
						OR slug LIKE ?
						OR body LIKE ?
						OR meta_description LIKE ?";

		$stmt = $this->conn->prepare($query);

		// sanitize
		$search_term = "%{$search_term}%";
		$search_term=htmlspecialchars(strip_tags($search_term));

		// bind limit clause variables
		$stmt->bindParam(1, $search_term);
		$stmt->bindParam(2, $search_term);
		$stmt->bindParam(3, $search_term);
		$stmt->bindParam(4, $search_term);

		$stmt->execute();

		$num = $stmt->rowCount();

		return $num;
	}


	// used when filling up the update page form
	function readOne(){

		$query = "SELECT p.id, p.title, p.slug, p.body, p.meta_description, p.status,
							p.featured_image, p.user_id, p.created, u.firstname, u.lastname
				FROM " . $this->table_name . " p
					LEFT JOIN users u
						ON p.user_id = u.id
				WHERE p.id = ?
				LIMIT 0,1";

		$stmt = $this->conn->prepare( $query );
		$stmt->bindParam(1, $this->id);
		$stmt->execute();

		$row = $stmt->fetch(PDO::FETCH_ASSOC);

		$this->title = $row['title'];
		$this->slug = $row['slug'];
		$this->body = $row['body'];
		$this->meta_description = $row['meta_description'];
		$this->status = $row['status'];
		$this->featured_image = $row['featured_image'];
		$this->user_id = $row['user_id'];
	}

	// used when filling up the update page form
	function readOneBySlug(){

		$query = "SELECT p.id, p.title, p.slug, p.body, p.meta_description, p.status,
							p.featured_image, p.user_id, p.created, u.firstname, u.lastname
				FROM " . $this->table_name . " p
					LEFT JOIN users u
						ON p.user_id = u.id
				WHERE p.slug = :slug
				LIMIT 0,1";

		$stmt = $this->conn->prepare( $query );
		$stmt->bindParam(':slug', $this->slug);
		$stmt->execute();

		$row = $stmt->fetch(PDO::FETCH_ASSOC);

		$this->id = $row['id'];
		$this->title = $row['title'];
		$this->slug = $row['slug'];
		$this->body = $row['body'];
		$this->meta_description = $row['meta_description'];
		$this->status = $row['status'];
		$this->featured_image = $row['featured_image'];
		$this->firstname = $row['firstname'];
		$this->lastname = $row['lastname'];
		$this->created = $row['created'];
	}

	// update the page
	function update(){

		$query = "UPDATE " . $this->table_name . "
				SET
					title=:title,
					slug=:slug,
					body=:body,
					meta_description=:meta_description,
					featured_image=:featured_image,
					status=:status
				WHERE id = :id";

		$stmt = $this->conn->prepare($query);

		// sanitize
		$this->title=htmlspecialchars(strip_tags($this->title));
		$this->slug=htmlspecialchars(strip_tags($this->slug));
		$this->body=htmlspecialchars($this->body);
		$this->meta_description=htmlspecialchars(strip_tags($this->meta_description));
		$this->featured_image=htmlspecialchars(strip_tags($this->featured_image));
		$this->status=htmlspecialchars(strip_tags($this->status));
		$this->id=htmlspecialchars(strip_tags($this->id));

		// bind values
		$stmt->bindParam(':title', $this->title);
		$stmt->bindParam(':slug', $this->slug);
		$stmt->bindParam(':body', $this->body);
		$stmt->bindParam(':meta_description', $this->meta_description);
		$stmt->bindParam(':featured_image', $this->featured_image);
		$stmt->bindParam(':status', $this->status);
		$stmt->bindParam(':id', $this->id);

		// execute the query
		if($stmt->execute()){
			return true;
		}else{
			return false;
		}
	}

	// update the page
	function updateWithoutFeaturedImage(){

		$query = "UPDATE " . $this->table_name . "
				SET
					title=:title,
					slug=:slug,
					body=:body,
					meta_description=:meta_description,
					status=:status
				WHERE id = :id";

		$stmt = $this->conn->prepare($query);

		// sanitize
		$this->title=htmlspecialchars(strip_tags($this->title));
		$this->slug=htmlspecialchars(strip_tags($this->slug));
		$this->body=htmlspecialchars($this->body);
		$this->meta_description=htmlspecialchars(strip_tags($this->meta_description));
		$this->status=htmlspecialchars(strip_tags($this->status));
		$this->id=htmlspecialchars(strip_tags($this->id));

		// bind values
		$stmt->bindParam(':title', $this->title);
		$stmt->bindParam(':slug', $this->slug);
		$stmt->bindParam(':body', $this->body);
		$stmt->bindParam(':meta_description', $this->meta_description);
		$stmt->bindParam(':status', $this->status);
		$stmt->bindParam(':id', $this->id);

		// execute the query
		if($stmt->execute()){
			return true;
		}else{
			return false;
		}
	}

	// delete the page
	function delete(){

		$query = "DELETE FROM " . $this->table_name . " WHERE id = ?";

		$stmt = $this->conn->prepare($query);
		$stmt->bindParam(1, $this->id);

		if($result = $stmt->execute()){
			return true;
		}else{
			return false;
		}
	}

}
?>
